!---------------------------------------------------------------------!
!                                PHAML                                !
!                                                                     !
! The Parallel Hierarchical Adaptive MultiLevel code for solving      !
! linear elliptic partial differential equations of the form          !
! (PUx)x + (QUy)y + RU = F on 2D polygonal domains with mixed         !
! boundary conditions, and eigenvalue problems where F is lambda*U.   !
!                                                                     !
! PHAML is public domain software.  It was produced as part of work   !
! done by the U.S. Government, and is not subject to copyright in     !
! the United States.                                                  !
!                                                                     !
!     William F. Mitchell                                             !
!     Applied and Computational Mathematics Division                  !
!     National Institute of Standards and Technology                  !
!     william.mitchell@nist.gov                                       !
!     http://math.nist.gov/phaml                                      !
!                                                                     !
!---------------------------------------------------------------------!

!----------------------------------------------------
! This file contains the main program supplied by the user.
!
! This version solves a parabolic equation.  See README.parabolic for
! a description of the implicit time stepping scheme.
! This is to illustrate the use of keep_linsys.  The grid is only adapted
! once every so many time steps, and the linear system is kept between them.
!
!----------------------------------------------------

!       ------------
program phaml_master
!       ------------

!----------------------------------------------------
! This is the main program.
!----------------------------------------------------

!----------------------------------------------------
! Modules used are:

use global
use phaml
use phaml_user_mod
!----------------------------------------------------

implicit none

!----------------------------------------------------
! Local variables:

integer :: nproc,nelem,ref_freq,it,task
real(my_real) :: finalt
type(phaml_solution_type) :: soln

!----------------------------------------------------
! Begin executable code

nelem = 1000
print *,"number of elements for the grid (for example, 1000)?"
read *,nelem
finalt = 1.0_my_real
print *,"final time (for example, 1.0)?"
read *,finalt
deltat = .01_my_real
print *,"time step (for example, .01)?"
read *,deltat
ref_freq = 5
print *,"number of time steps between adaptive refinements (for example, 5)?"
read *,ref_freq

nproc = 1
print *,"number of processors?"
read *,nproc

! create the phaml_solution variables

it = 0
t = 0.0_my_real
call phaml_create(soln,nproc,update_umod=.true., &
!                  spawn_form=DEBUG_SLAVE,debug_command="gdb", &
                  draw_grid_who = MASTER)
! set the initial condition

call phaml_solve_pde(soln,                      &
                     max_elem=nelem,            &
                     task=SET_INITIAL,          &
                     pde_has_first_order_terms=.false., &
                     pde_has_cross_derivative=.false., &
                     laplacian_operator=.false., &
                     refterm=ONE_REF_HALF_ERRIND, &
                     error_estimator=INITIAL_CONDITION, &
                     print_header_who=NO_ONE,   &
                     print_trailer_who=NO_ONE,  &
                     draw_grid_when=FINAL)

! give time to adjust the graphics view

print *,"adjust graphics view and press return"
read *

! solve the equation until the final time is reached.

! task is BALANCE_REFINE_SOLVE when the time-iteration is a multiple of
! ref_freq, and SOLVE_ONLY on the other time steps.  keep_linear_system=.true.
! keeps the linear system from one time step to the next, rather than the
! default behavior of destroying it before returning and recreating it on
! the next call.  Although the matrix is reused, the right hand side is always
! recomputed.  Also, when the grid is adapted, the entire linear system is
! automatically destroyed and recreated.  And the final linear system is
! destroyed by phaml_destroy.

do
   it = it + 1
   t = it*deltat
   if (t > finalt) exit
   if (mod(it,ref_freq) == 0) then
      task = BALANCE_REFINE_SOLVE
   else
      task = SOLVE_ONLY
   endif
   call update_usermod(soln)
   call phaml_copy_soln_to_old(soln)
   call phaml_solve_pde(soln,                    &
                        max_refsolveloop=1,      &
                        task=task,               &
                        solve_init=.false.,      &
                        keep_linear_system=.true., &
                        refterm=KEEP_NELEM,      &
                        max_elem=nelem,          &
                        mg_cycles=10,            &
                        draw_grid_when=FINAL ,   &
                        print_warnings=.false.,  &
                        print_header_who=NO_ONE, &
                        print_trailer_who=NO_ONE)
   print *,"time = ",t
end do

print *,"press return to terminate program"
read *

call phaml_destroy(soln)

end program phaml_master
